# <trigger>
#	<usage>perl checkChildAttr.pl reportName attributeName attributeValue</usage>
#	<version>4.4</version>
#	<description>親子関係レポートを実行してすべての子オブジェクトが同じ属性値をもつことを確認します。
#                    その属性についてすべての子が同じ値をもつ場合は、CR の遷移が許可されます。
#                    そうでない場合は CR は遷移できません。ALM ライフサイクルではこのトリガーを使用して BCR が遷移可能かどうか確認します。
#					 実行レポートは Data_Report_BCR_RCP です。
#					 すべての RCP について、属性 ready_for_final_review が「TRUE」に設定されているかをチェックします。
#                    チェック対象の属性がレポート定義の 2 番目の [PROBLEM_DEF] に定義されていることを確認してください。
#                    通常このトリガーは、事前条件トリガーとして実行されます。
#   </description>
#	<argument1>
#		<name>reportName</name>
#		<required>true</required>
#		<type></type>
#		<description>トリガー中に実行されるレポートの名称。</description>
#   </argument1>
#	<argument2>
#		<name>attributeName</name>
#		<required>true</required>
#		<type></type>
#		<description>チェック対象の属性の名称。</description>
#   </argument2>
#	<argument3>
#		<name>attributeValue</name>
#		<required>true</required>
#		<type></type>
#		<description>期待される属性値。</description>
#   </argument3>
# </trigger>

use strict;
use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $report = shift;
my $attrName = shift;
my $attrValue = shift;

#Create a new TriggerParser object, passing in the location of the trigger xml document.
my $trigger = new ChangeSynergy::TriggerParser($ARGV[-1]);

#Create a new csapi object.
my $csapi = new ChangeSynergy::csapi();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $
				      trigger->get_token(), $trigger->get_database());

#Create a query string and then call the api to get a data report.
my $queryString = "problem_number='" . $trigger->get_object_id() . "'";
my $data = $csapi->QueryData($user, $report, $queryString, undef, undef, undef);

#Empty array which will hold the problem numbers for the problems to be transitioned.
my @problemsToTest = ();

#For each top level csapi_cobject_vector_postion.
for(my $i = 0; $i < $data->getDataSize(); $i++)
{
	#Get the child data, this is the associated data, the sub-report data.
	my $childData = $data->getDataObject($i)->getChildData();

	#For each csapi_object_vector_position under the csapi_cobject_vector_assoc tag.
	for(my $j = 0; $j < $childData->getDataSize(); $j++)
	{
		#Get the object vector
		my $objVec = $childData->getDataObject($j);

		# If ready_for_final_review isn't true on this child, then fail.
		if ($objVec->getDataObjectByName($attrName)->getValue() ne "\L$attrValue")
		{
			die "Cannot transition: RCP " . $objVec->getDataObjectByName("problem_number")->getValue() . " is not ready for final review\n";
		}
	}
}

# If we got through all of the childern 
exit(0);