/*
<trigger>
	<usage>bsf ecp_update_duplicate_status_self.js</usage>
	<version>5.0</version>
	<description>
		現在の CR を、その CR の重複元の CR の状態に基づいて遷移させます。
		重複 CR とは、現在の CR と「重複」関係を使用して関連付けられた CR です。
		ターゲット状態は、元の CR の状態に「重複」という文字列が付いた状態です。
		例えば、「分析中」の場合は「分析中と重複」になります。</description>
	</description>
</trigger>
*/


// Import packages
importPackage(Packages.com.telelogic.cs.api);

// Script specific global variables
const script_name = 'ecp_update_duplicate_status_self.js';
const script_version = '20080604';
const script_usage = 'ecp_update_duplicate_status_self.js';
const script_example = 'ecp_update_duplicate_status_self.js';
const log_header = 'Script bsf ' + script_name + ': ';
const script_debug = false;

// Get objects
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');
var options = OptionParser.parse(args);

if (script_debug) 
{
	log.info(log_header);
	log.info('    Version: ' + script_version);
	log.info('    Arguments: ' + args.join(' '));
	log.info('    Current CR: ' + event.objectId);
}

if (checkArguments(0, [], []))
{
	// Find original CR
	const relationship = 'duplicate';
	const crstate_attr_name = 'crstatus'
	var attribute_names = [crstate_attr_name];
	var items = change.getRelatedTo(event.triggerUser.token, event.objectId, relationship, attribute_names);
	
	if (items.length == 0)
	{
		log.error('CR ' + event.objectId + 'is not a duplicate to another CR');
	}
	else
	{
		if (script_debug)
		{
			log.info(log_header + 'Found original CR ' + items[0].crid);
		}

		// Get original CRs status
		const original_state = items[0].getAttribute(crstate_attr_name);
		
		if (isAttributeNull(original_state))
		{
			log.warn(log_header + 'Original status has no value');
		}
		
		if (script_debug) 
		{
			log.info(log_header + 'Original CR has status \'' + original_state + '\'');
		}

		// Get to state
		const prefix = 'duplicate_';
		const to_state = prefix + original_state; 
		
		if (script_debug) 
		{
			log.info(log_header + 'Target status is \'' + to_state + '\'');
		}

		// Transition CR
		try
		{
			change.transition(event.triggerUser.token, event.objectId, to_state, []);
		}
		catch (error)
		{
			var from_state = change.collectAttributes(event.triggerUser.token, event, ['crstatus']).get('crstatus').toString();
			var message = change.getMessage('errorTransitioningCRFromXtoStateYerrorZ', [from_state, to_state, error.message || error.toString()]);
			log.error(log_header + message);
			event.fail(message);
		}
	}
}

/**
 * Checks the script arguments:
 * - Check argument count
 * - Check that all specified options are valid
 * - Check that all required options are specified
 *
 * Returns: true, if succesfull. false otherwise
 */
function checkArguments(arg_count, allowed_options, required_options)
{
	var message;

	// Check argument count
	if (options.args.length < arg_count)
		message = 'Insufficient arguments to run the script (' + options.args.length +' arguments)';

	// Check that all specified options are valid
	if (!message)
	{
		var all_options = options.getOptions();
		var i = all_options.keySet().iterator();
		
		while (!message && i.hasNext())
		{
			var option = i.next();
			var is_valid = false;
			
			for (var j = 0; j < allowed_options.length; j++)
			{
				if (option == allowed_options[j])
				{
					is_valid = true;
					break;
				}
			}

			if (!is_valid)
			{
				message = 'Invalid option \'' + option + '\'';
			}
		}
	}

	// Check that all required options are specified
	if (!message) 
	{
		for (var i = 0; i < required_options.length; i++)
		{
			var option = options.getOption(required_options[i]);
			
			if (!option)
			{
				message = 'Missing option \'' + required_options[i] + '\'';
			}
		}
	}

	if (message)
	{
		log.error(log_header);
		log.error('   ' + message);
		log.error('   Usage: ' + script_usage);
		log.error('   Example: ' + script_example);
		return false;
	}

	return true;
}


/**
 * Checks, if an attribute has no value set.
 *
 * The function is necessary, because Attribute.toString() returns 'null' instead of null. By
 * returning 'null' it cannot be distinguished between an without a value and one with the 
 * value 'null'.
 *
 * Returns: True, if the attriute has no value set.
 */
function isAttributeNull(attribute)
{
	if (((!attribute) || (attribute.type == Attribute.Type.STRING) && (attribute.getStringValue() == null))
		|| ((attribute.type == Attribute.Type.DATE) && (attribute.getDateValue() == null))
		|| ((attribute.type == Attribute.Type.BOOLEAN) && (attribute.getBooleanValue() == null)))
	{
		return true;
	}

	return false;
}
