// Copyright (c) 2002-2007 Telelogic AB
// All rights reserved.

/*
<trigger>
	<usage>bsf copy_attributes_to_associated_tasks.js attributes</usage>
	<version>5.0</version>
	<description>
		在转变期间将属性值从一个 CR 复制到与其关联的任务。
	</description>
	<argument1>
		<name>attributes</name>
		<required>yes</required>
		<type>string</type>
		<description>从要转变的 CR 复制到与其关联的任务的一个或多个以空格分隔的属性。例如，自变量“release priority”将设置关联任务的发行版和优先级属性，以与该 CR 的属性匹配。仅当在 CR 与任务之间使用相同的属性值时，这才有用。</description>
	</argument1>
	<argument2>
		<name>-ifnotstate state_names</name>
		<required>no</required>
		<type>string</type>
		<description>要检查的一个或多个以空格分隔的状态名称。不会将属性复制到处于指定状态的任何关联任务。例如，自变量“-ifnotstate completed excluded”不会修改任何已完成和已排除的任务；但仍将修改 CR 上其他未完成或未排除的任务。</description>
	</argument2>
</trigger>
*/
importPackage(Packages.com.telelogic.cs.api);

var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');

if (args.length < 1)
{
	log.error('Could not copy attributes to associated tasks: no attributes were given.');
	log.error('Usage: copy_attributes_to_associated_tasks.js attribute_name [attribute_name...] [-ifnotstate state1_to_skip [state2_to_skip...]]');
	log.error('Example: copy_attributes_to_associated_tasks.js resolver -ifnotstate completed');
}
else
{

	var allArgs = args;
	var attributesInArgs = allArgs;
	var skipStateSet;

	for (var i = 0; i < allArgs.length; i++)
	{
		if (allArgs[i].indexOf("-ifnotstate") == 0)
		{
			attributesInArgs = allArgs.slice(0, i);
			skipStateSet = allArgs.slice(i+1);
		}
	}
	
	// lookup the 'status' too if we want to skip tasks in a given state and need to compare it
	var attributeNames = skipStateSet ? ['status', 'from_db_path'] : ['from_db_path'];
	var tasks = change.getAssociatedTasks(event.triggerUser.token, event.objectId, attributeNames);

	if (tasks.length > 0)
	{
		var crAttributes = change.collectAttributes(event.triggerUser.token, event, attributesInArgs);

		var copiedAttributes = [];
		
		// gather only the attributes mentioned as args
		for (var i = 0; i < attributesInArgs.length; i++)
		{
			var name = attributesInArgs[i];
			var crAttribute = crAttributes.get(name);
			// gather that attribute if it was set on the CR. Otherwise, it's ignored
			if (crAttribute) copiedAttributes.push(crAttribute);
		}
		
		var doneTasks = {};
		var tasksToErrors = {};

		for (var i = 0; i < tasks.length; i++) 
		{
			var currentTask = tasks[i];
			
			var found = false;
			if (skipStateSet)
			{
				for (var j = 0; j < skipStateSet.length; j++)
				{
					if (currentTask.getAttribute('status').stringValue.equals(skipStateSet[j])) found = true;
				}
				if (found) 
				{
					continue;
				}
			}
			
			if (doneTasks[currentTask.taskId]) continue;

			// in central server mode, passing in a from_db_path will override the context database.
			// This lets up modify the task whereever it came from.
			var from = currentTask.getAttribute('from_db_path');

			var taskAttributes = from ?
				copiedAttributes.concat([from]) :
				copiedAttributes;

			try
			{
				change.setTaskAttributes(event.triggerUser.token, currentTask.taskId, taskAttributes);

				// the task transitioned in some database. Don't warn about it anymore. In central server mode
				// we may get DCM copies of tasks from multiple databases--we only need to transition the
				// task successfully in one of them (the modifiable_in DB).
				doneTasks[currentTask.taskId] = true;
				delete tasksToErrors[currentTask.taskId];
			}
			catch (error)
			{
				tasksToErrors[currentTask.taskId] =
					change.getMessage('couldNotCopyAttributesToTaskXErrorX', [currentTask.taskId, error.message || error.toString()]);
			}
		}
		
		for (var task in tasksToErrors)
		{
			event.fail(tasksToErrors[task]);
		}
	}
}