#  <trigger>
# 	<usage>perl subscribe.pl list_attribute userid_attribute</usage>
# 	<version>4.3</version>
# 	<description>将 userid_attribute 的内容添加到包含在 list_attribute 中的用户列表。该操作与通知触发器一起使用，以将信息发送至 CR 的预订用户列表。
#		通常，通过将此用作转变后/修改触发器以自动将提交者等添加到 CR 的通知，来实现此操作。
#   </description>
# 	<argument1>
# 		<name>list_attribute</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>包含要添加到的用户标识列表的属性的名称。</description>
#        </argument1>
# 	<argument2>
# 		<name>userid_attribute</name>
# 		<required>true</required>
# 		<type></type>
# 		<description>包含应添加到 list_attribute 的用户标识的属性的名称。</description>
#        </argument2>
#  </trigger>

use utf8;
use ChangeSynergy::TriggerParser;
use ChangeSynergy::csapi;

my $list_attribute = shift;
my $userid_attribute = shift;

my $triggerFile = shift;

#Create a new TriggerParser object
my $trigger = new ChangeSynergy::TriggerParser($triggerFile);

#Create new csapi object
my $csapi = new ChangeSynergy::csapi();
my $problem_number   = $trigger->get_object_id();

#setup the connection
$csapi->setUpConnection($trigger->get_base_url());

#create a user object
my $user = new ChangeSynergy::apiUser($trigger->get_user(), "password", $trigger->get_role(), $trigger->get_token(), $trigger->get_database());

my %trigger_attributes = $trigger->get_attributes();

my $userid = $trigger_attributes{$userid_attribute};
my $listvalue = "";

# add the specified user field to the list if they are not already on it.
$listvalue = $trigger_attributes{$list_attribute} || '';

unless ($listvalue =~ /$userid/)
{
	my $cr = $csapi->GetCRData($user, $problem_number, $list_attribute);
	
	if (length($listvalue) == 0) 
	{
		$listvalue = "<?xml version='1.0' encoding='US-ASCII'?><subscription></subscription>";
	}
	
	my $subscriptionStr = "</subscription>";
	my $subscription_list =
	    substr($listvalue, 0, index($listvalue, $subscriptionStr))
	  . "<subscriber><username>"
	  . $userid
	  . "</username><realname>"
	  . "</realname><email>"
	  . "</email><uid></uid><action>1</action></subscriber>"
	  . $subscriptionStr;
	  
	$cr->getDataObjectByName($list_attribute)->setValue($subscription_list);
	$csapi->ModifyCR($user, $cr);
	
	print "Subscribed $userid to $list_attribute \n";
}
else
{
	print "$userid is already subscribed to $list_attribute on cr $problem_number\n";
}

exit 0;
